@@ -37,6 +37,7 @@
 #include <vcsbase/vcsbaseconstants.h>
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
+#include <utils/optional.h>
 #include <utils/qtcassert.h>
 
 #include <QDir>
@@ -49,8 +50,6 @@
 
 namespace Core {
 
-typedef QList<IVersionControl *> VersionControlList;
-
 #if defined(WITH_TESTS)
 const char TEST_PREFIX[] = "/8E3A9BA0-0B97-40DF-AEC1-2BDF9FC9EDBE/";
 #endif
@@ -63,53 +62,29 @@ class VcsManagerPrivate
 public:
     class VcsInfo {
     public:
+        VcsInfo() = default;
         VcsInfo(IVersionControl *vc, const QString &tl) :
             versionControl(vc), topLevel(tl)
         { }
+        VcsInfo(const VcsInfo &other) = default;
 
         bool operator == (const VcsInfo &other) const
         {
-            return versionControl == other.versionControl &&
-                    topLevel == other.topLevel;
+            return versionControl == other.versionControl && topLevel == other.topLevel;
         }
 
-        IVersionControl *versionControl;
+        IVersionControl *versionControl = nullptr;
         QString topLevel;
     };
 
-    VcsManagerPrivate() : m_unconfiguredVcs(0), m_cachedAdditionalToolsPathsDirty(true)
-    { }
-
-    ~VcsManagerPrivate()
+    Utils::optional<VcsInfo> findInCache(const QString &dir)
     {
-        qDeleteAll(m_vcsInfoList);
-    }
+        QTC_ASSERT(QDir(dir).isAbsolute(), return Utils::nullopt);
+        QTC_ASSERT(!dir.endsWith(QLatin1Char('/')), return Utils::nullopt);
+        QTC_ASSERT(QDir::fromNativeSeparators(dir) == dir, return Utils::nullopt);
 
-    VcsInfo *findInCache(const QString &dir)
-    {
-        QTC_ASSERT(QDir(dir).isAbsolute(), return 0);
-        QTC_ASSERT(!dir.endsWith(QLatin1Char('/')), return 0);
-        QTC_ASSERT(QDir::fromNativeSeparators(dir) == dir, return 0);
-
-        const QMap<QString, VcsInfo *>::const_iterator it = m_cachedMatches.constFind(dir);
-        if (it != m_cachedMatches.constEnd())
-            return it.value();
-        return 0;
-    }
-
-    VcsInfo *findUpInCache(const QString &directory)
-    {
-        VcsInfo *result = 0;
-        const QChar slash = QLatin1Char('/');
-        // Split the path, trying to find the matching repository. We start from the reverse
-        // in order to detected nested repositories correctly (say, a git checkout under SVN).
-        for (int pos = directory.size() - 1; pos >= 0; pos = directory.lastIndexOf(slash, pos) - 1) {
-            const QString directoryPart = directory.left(pos);
-            result = findInCache(directoryPart);
-            if (result != 0)
-                break;
-        }
-        return result;
+        const auto it = m_cachedMatches.constFind(dir);
+        return it == m_cachedMatches.constEnd() ? Utils::nullopt : Utils::make_optional(it.value());
     }
 
     void clearCache()
@@ -139,24 +114,10 @@ public:
                    || topLevel == dir || topLevel.isEmpty(), return);
         QTC_ASSERT((topLevel.isEmpty() && !vc) || (!topLevel.isEmpty() && vc), return);
 
-        VcsInfo *newInfo = new VcsInfo(vc, topLevel);
-        bool createdNewInfo(true);
-        // Do we have a matching VcsInfo already?
-        foreach (VcsInfo *i, m_vcsInfoList) {
-            if (*i == *newInfo) {
-                delete newInfo;
-                newInfo = i;
-                createdNewInfo = false;
-                break;
-            }
-        }
-        if (createdNewInfo)
-            m_vcsInfoList.append(newInfo);
-
         QString tmpDir = dir;
         const QChar slash = QLatin1Char('/');
         while (tmpDir.count() >= topLevel.count() && !tmpDir.isEmpty()) {
-            m_cachedMatches.insert(tmpDir, newInfo);
+            m_cachedMatches.insert(tmpDir, VcsInfo(vc, topLevel));
             // if no vc was found, this might mean we're inside a repo internal directory (.git)
             // Cache only input directory, not parents
             if (!vc)
@@ -169,16 +130,16 @@ public:
         }
     }
 
-    QMap<QString, VcsInfo *> m_cachedMatches;
-    QList<VcsInfo *> m_vcsInfoList;
-    IVersionControl *m_unconfiguredVcs;
+    QList<IVersionControl *> m_versionControlList;
+    QMap<QString, VcsInfo> m_cachedMatches;
+    IVersionControl *m_unconfiguredVcs = nullptr;
 
     QStringList m_cachedAdditionalToolsPaths;
-    bool m_cachedAdditionalToolsPathsDirty;
+    bool m_cachedAdditionalToolsPathsDirty = true;
 };
 
-static VcsManagerPrivate *d = 0;
-static VcsManager *m_instance = 0;
+static VcsManagerPrivate *d = nullptr;
+static VcsManager *m_instance = nullptr;
 
 VcsManager::VcsManager(QObject *parent) :
    QObject(parent)
@@ -191,10 +152,16 @@ VcsManager::VcsManager(QObject *parent) :
 
 VcsManager::~VcsManager()
 {
-    m_instance = 0;
+    m_instance = nullptr;
     delete d;
 }
 
+void VcsManager::addVersionControl(IVersionControl *vc)
+{
+    QTC_ASSERT(!d->m_versionControlList.contains(vc), return);
+    d->m_versionControlList.append(vc);
+}
+
 VcsManager *VcsManager::instance()
 {
     return m_instance;
@@ -213,9 +180,9 @@ void VcsManager::extensionsInitialized()
     }
 }
 
-QList<IVersionControl *> VcsManager::versionControls()
+const QList<IVersionControl *> VcsManager::versionControls()
 {
-    return ExtensionSystem::PluginManager::getObjects<IVersionControl>();
+    return d->m_versionControlList;
 }
 
 IVersionControl *VcsManager::versionControl(Id id)
@@ -249,7 +216,7 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     if (inputDirectory.isEmpty()) {
         if (topLevelDirectory)
             topLevelDirectory->clear();
-        return 0;
+        return nullptr;
     }
 
     // Make sure we an absolute path:
@@ -258,7 +225,7 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     if (directory[0].isLetter() && directory.indexOf(QLatin1Char(':') + QLatin1String(TEST_PREFIX)) == 1)
         directory = directory.mid(2);
 #endif
-    VcsManagerPrivate::VcsInfo *cachedData = d->findInCache(directory);
+    auto cachedData = d->findInCache(directory);
     if (cachedData) {
         if (topLevelDirectory)
             *topLevelDirectory = cachedData->topLevel;
@@ -282,12 +249,12 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
     });
 
     if (allThatCanManage.isEmpty()) {
-        d->cache(0, QString(), directory); // register that nothing was found!
+        d->cache(nullptr, QString(), directory); // register that nothing was found!
 
         // report result;
         if (topLevelDirectory)
             topLevelDirectory->clear();
-        return 0;
+        return nullptr;
     }
 
     // Register Vcs(s) with the cache
@@ -325,11 +292,11 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
         if (isVcsConfigured) {
             if (curDocument && d->m_unconfiguredVcs == versionControl) {
                 curDocument->infoBar()->removeInfo(vcsWarning);
-                d->m_unconfiguredVcs = 0;
+                d->m_unconfiguredVcs = nullptr;
             }
             return versionControl;
         } else {
-            InfoBar *infoBar = curDocument ? curDocument->infoBar() : 0;
+            InfoBar *infoBar = curDocument ? curDocument->infoBar() : nullptr;
             if (infoBar && infoBar->canInfoBeAdded(vcsWarning)) {
                 InfoBarEntry info(vcsWarning,
                                   tr("%1 repository was detected but %1 is not configured.")
@@ -343,7 +310,7 @@ IVersionControl* VcsManager::findVersionControlForDirectory(const QString &input
 
                 infoBar->addInfo(info);
             }
-            return 0;
+            return nullptr;
         }
     }
     return versionControl;
@@ -359,9 +326,10 @@ QString VcsManager::findTopLevelForDirectory(const QString &directory)
 QStringList VcsManager::repositories(const IVersionControl *vc)
 {
     QStringList result;
-    foreach (const VcsManagerPrivate::VcsInfo *vi, d->m_vcsInfoList)
-        if (vi->versionControl == vc)
-            result.push_back(vi->topLevel);
+    for (auto it = d->m_cachedMatches.constBegin(); it != d->m_cachedMatches.constEnd(); ++it) {
+        if (it.value().versionControl == vc)
+            result.append(it.value().topLevel);
+    }
     return result;
 }
 
@@ -496,31 +464,6 @@ const char ID_VCS_B[] = "B";
 
 typedef QHash<QString, QString> FileHash;
 
-template<class T>
-class ObjectPoolGuard
-{
-public:
-    ObjectPoolGuard(T *watch) : m_watched(watch)
-    {
-        ExtensionSystem::PluginManager::addObject(watch);
-    }
-
-    explicit operator bool() { return m_watched; }
-    bool operator !() { return !m_watched; }
-    T &operator*() { return *m_watched; }
-    T *operator->() { return m_watched; }
-    T *value() { return m_watched; }
-
-    ~ObjectPoolGuard()
-    {
-        ExtensionSystem::PluginManager::removeObject(m_watched);
-        delete m_watched;
-    }
-
-private:
-    T *m_watched;
-};
-
 static FileHash makeHash(const QStringList &list)
 {
     FileHash result;
@@ -550,62 +493,56 @@ void CorePlugin::testVcsManager_data()
     QTest::addColumn<QStringList>("results");
 
     QTest::newRow("A and B next to each other")
-            << (QStringList()
-                << QLatin1String("a:a") << QLatin1String("a/1:a") << QLatin1String("a/2:a")
-                << QLatin1String("a/2/5:a") << QLatin1String("a/2/5/6:a"))
-            << (QStringList()
-                << QLatin1String("b:b") << QLatin1String("b/3:b") << QLatin1String("b/4:b"))
-            << (QStringList()
-                << QLatin1String(":::-") // empty directory to look up
-                << QLatin1String("c:::*") // Neither in A nor B
-                << QLatin1String("a:a:A:*") // in A
-                << QLatin1String("b:b:B:*") // in B
-                << QLatin1String("b/3:b:B:*") // in B
-                << QLatin1String("b/4:b:B:*") // in B
-                << QLatin1String("a/1:a:A:*") // in A
-                << QLatin1String("a/2:a:A:*") // in A
-                << QLatin1String(":::-") // empty directory to look up
-                << QLatin1String("a/2/5/6:a:A:*") // in A
-                << QLatin1String("a/2/5:a:A:-") // in A (cached from before!)
-                // repeat: These need to come from the cache now:
-                << QLatin1String("c:::-") // Neither in A nor B
-                << QLatin1String("a:a:A:-") // in A
-                << QLatin1String("b:b:B:-") // in B
-                << QLatin1String("b/3:b:B:-") // in B
-                << QLatin1String("b/4:b:B:-") // in B
-                << QLatin1String("a/1:a:A:-") // in A
-                << QLatin1String("a/2:a:A:-") // in A
-                << QLatin1String("a/2/5/6:a:A:-") // in A
-                << QLatin1String("a/2/5:a:A:-") // in A
-                );
+            << QStringList({"a:a", "a/1:a", "a/2:a", "a/2/5:a", "a/2/5/6:a"})
+            << QStringList({"b:b", "b/3:b", "b/4:b"})
+            << QStringList({":::-",          // empty directory to look up
+                            "c:::*",         // Neither in A nor B
+                            "a:a:A:*",       // in A
+                            "b:b:B:*",       // in B
+                            "b/3:b:B:*",     // in B
+                            "b/4:b:B:*",     // in B
+                            "a/1:a:A:*",     // in A
+                            "a/2:a:A:*",     // in A
+                            ":::-",          // empty directory to look up
+                            "a/2/5/6:a:A:*", // in A
+                            "a/2/5:a:A:-",   // in A (cached from before!)
+                            // repeat: These need to come from the cache now:
+                            "c:::-",         // Neither in A nor B
+                            "a:a:A:-",       // in A
+                            "b:b:B:-",       // in B
+                            "b/3:b:B:-",     // in B
+                            "b/4:b:B:-",     // in B
+                            "a/1:a:A:-",     // in A
+                            "a/2:a:A:-",     // in A
+                            "a/2/5/6:a:A:-", // in A
+                            "a/2/5:a:A:-"    // in A
+                });
     QTest::newRow("B in A")
-            << (QStringList()
-                << QLatin1String("a:a") << QLatin1String("a/1:a") << QLatin1String("a/2:a")
-                << QLatin1String("a/2/5:a") << QLatin1String("a/2/5/6:a"))
-            << (QStringList()
-                << QLatin1String("a/1/b:a/1/b") << QLatin1String("a/1/b/3:a/1/b")
-                << QLatin1String("a/1/b/4:a/1/b") << QLatin1String("a/1/b/3/5:a/1/b")
-                << QLatin1String("a/1/b/3/5/6:a/1/b"))
-            << (QStringList()
-                << QLatin1String("a:a:A:*") // in A
-                << QLatin1String("c:::*") // Neither in A nor B
-                << QLatin1String("a/3:::*") // Neither in A nor B
-                << QLatin1String("a/1/b/x:::*") // Neither in A nor B
-                << QLatin1String("a/1/b:a/1/b:B:*") // in B
-                << QLatin1String("a/1:a:A:*") // in A
-                << QLatin1String("a/1/b/../../2:a:A:*") // in A
-                );
+            << QStringList({"a:a", "a/1:a", "a/2:a", "a/2/5:a", "a/2/5/6:a"})
+            << QStringList({"a/1/b:a/1/b", "a/1/b/3:a/1/b", "a/1/b/4:a/1/b", "a/1/b/3/5:a/1/b",
+                            "a/1/b/3/5/6:a/1/b"})
+            << QStringList({"a:a:A:*",            // in A
+                            "c:::*",              // Neither in A nor B
+                            "a/3:::*",            // Neither in A nor B
+                            "a/1/b/x:::*",        // Neither in A nor B
+                            "a/1/b:a/1/b:B:*",    // in B
+                            "a/1:a:A:*",          // in A
+                            "a/1/b/../../2:a:A:*" // in A
+                });
     QTest::newRow("A and B") // first one wins...
-            << (QStringList() << QLatin1String("a:a") << QLatin1String("a/1:a") << QLatin1String("a/2:a"))
-            << (QStringList() << QLatin1String("a:a") << QLatin1String("a/1:a") << QLatin1String("a/2:a"))
-            << (QStringList() << QLatin1String("a/2:a:A:*"));
+            << QStringList({"a:a", "a/1:a", "a/2:a"})
+            << QStringList({"a:a", "a/1:a", "a/2:a"})
+            << QStringList({"a/2:a:A:*"});
 }
 
 void CorePlugin::testVcsManager()
 {
     // setup:
-    ObjectPoolGuard<TestVersionControl> vcsA(new TestVersionControl(ID_VCS_A, QLatin1String("A")));
-    ObjectPoolGuard<TestVersionControl> vcsB(new TestVersionControl(ID_VCS_B, QLatin1String("B")));
+    QList<IVersionControl *> orig = Core::d->m_versionControlList;
+    TestVersionControl *vcsA(new TestVersionControl(ID_VCS_A, QLatin1String("A")));
+    TestVersionControl *vcsB(new TestVersionControl(ID_VCS_B, QLatin1String("B")));
+
+    Core::d->m_versionControlList = {vcsA, vcsB};
 
     // test:
     QFETCH(QStringList, dirsVcsA);
@@ -649,7 +586,8 @@ void CorePlugin::testVcsManager()
     }
 
     // teardown:
-    // handled by guards
+    qDeleteAll(Core::d->m_versionControlList);
+    Core::d->m_versionControlList = orig;
 }
 
 } // namespace Internal
