@@ -25,6 +25,9 @@
 
 #pragma once
 
+#include "cppminimizableinfobars.h"
+#include "cppparsecontext.h"
+
 #include <cpptools/baseeditordocumentprocessor.h>
 #include <cpptools/cppcompletionassistprovider.h>
 #include <cpptools/cppmodelmanager.h>
@@ -55,10 +58,16 @@ public:
     void recalculateSemanticInfoDetached();
     CppTools::SemanticInfo recalculateSemanticInfo(); // TODO: Remove me
 
-    void setPreprocessorSettings(const CppTools::ProjectPart::Ptr &projectPart,
-                                 const QByteArray &defines);
+    void setPreferredParseContext(const QString &parseContextId);
+    void setExtraPreprocessorDirectives(const QByteArray &directives);
+
     void scheduleProcessDocument();
 
+    const MinimizableInfoBars &minimizableInfoBars() const;
+    ParseContextModel &parseContextModel();
+
+    QFuture<CppTools::CursorInfo> cursorInfo(const CppTools::CursorInfoParams &params);
+
 signals:
     void codeWarningsUpdated(unsigned contentsRevision,
                              const QList<QTextEdit::ExtraSelection> selections,
@@ -83,6 +92,8 @@ private:
     void onAboutToReload();
     void onReloadFinished();
 
+    void reparseWithPreferredParseContext(const QString &id);
+
     void processDocument();
 
     QByteArray contentsText() const;
@@ -90,9 +101,12 @@ private:
 
     CppTools::BaseEditorDocumentProcessor *processor();
     void resetProcessor();
-    void updatePreprocessorSettings();
+    void applyPreferredParseContextFromSettings();
+    void applyExtraPreprocessorDirectivesFromSettings();
     void releaseResources();
 
+    void showHideInfoBarAboutMultipleParseContexts(bool show);
+
     void initializeTimer();
 
 private:
@@ -112,6 +126,9 @@ private:
 
     // (Un)Registration in CppModelManager
     QScopedPointer<CppTools::CppEditorDocumentHandle> m_editorDocumentHandle;
+
+    MinimizableInfoBars m_minimizableInfoBars;
+    ParseContextModel m_parseContextModel;
 };
 
 } // namespace Internal
