@@ -23,15 +23,12 @@
 **
 ****************************************************************************/
 
-#ifndef IOSRUNFACTORIES_H
-#define IOSRUNFACTORIES_H
+#pragma once
 
 #include <projectexplorer/runconfiguration.h>
 #include <qmakeprojectmanager/qmakerunconfigurationfactory.h>
 
 namespace ProjectExplorer {
-class RunControl;
-class RunConfigWidget;
 class Target;
 class Node;
 } // namespace ProjectExplorer
@@ -69,23 +66,5 @@ private:
                                                  const QVariantMap &map) override;
 };
 
-class IosRunControlFactory : public ProjectExplorer::IRunControlFactory
-{
-    Q_OBJECT
-
-public:
-    explicit IosRunControlFactory(QObject *parent = 0);
-
-    bool canRun(ProjectExplorer::RunConfiguration *runConfiguration,
-                Core::Id mode) const override;
-    ProjectExplorer::RunControl *create(ProjectExplorer::RunConfiguration *runConfiguration,
-                       Core::Id mode,
-                       QString *errorMessage) override;
-private:
-    mutable QMap<Core::Id, QPointer<ProjectExplorer::RunControl> > m_activeRunControls;
-};
-
 } // namespace Internal
 } // namespace Ios
-
-#endif  // IOSRUNFACTORIES_H
