@@ -29,7 +29,6 @@
 #include "qmakeproject.h"
 #include "qmakebuildconfiguration.h"
 
-#include <coreplugin/coreicons.h>
 #include <coreplugin/variablechooser.h>
 #include <projectexplorer/localenvironmentaspect.h>
 #include <projectexplorer/runconfigurationaspects.h>
@@ -46,6 +45,7 @@
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 #include <utils/stringutils.h>
+#include <utils/utilsicons.h>
 
 #include <QCheckBox>
 #include <QComboBox>
@@ -76,67 +76,29 @@ static Utils::FileName pathFromId(Core::Id id)
 // QmakeRunConfiguration
 //
 
-DesktopQmakeRunConfiguration::DesktopQmakeRunConfiguration(Target *parent, Core::Id id) :
-    RunConfiguration(parent, id),
-    m_proFilePath(pathFromId(id))
+DesktopQmakeRunConfiguration::DesktopQmakeRunConfiguration(Target *target)
+    : RunConfiguration(target)
 {
     addExtraAspect(new LocalEnvironmentAspect(this, [](RunConfiguration *rc, Environment &env) {
                        static_cast<DesktopQmakeRunConfiguration *>(rc)->addToBaseEnvironment(env);
                    }));
-    addExtraAspect(new ArgumentsAspect(this, QStringLiteral("Qt4ProjectManager.Qt4RunConfiguration.CommandLineArguments")));
-    addExtraAspect(new TerminalAspect(this, QStringLiteral("Qt4ProjectManager.Qt4RunConfiguration.UseTerminal")));
-    addExtraAspect(new WorkingDirectoryAspect(this,
-                       QStringLiteral("Qt4ProjectManager.Qt4RunConfiguration.UserWorkingDirectory")));
+    addExtraAspect(new ArgumentsAspect(this, "Qt4ProjectManager.Qt4RunConfiguration.CommandLineArguments"));
+    addExtraAspect(new TerminalAspect(this, "Qt4ProjectManager.Qt4RunConfiguration.UseTerminal"));
+    addExtraAspect(new WorkingDirectoryAspect(this, "Qt4ProjectManager.Qt4RunConfiguration.UserWorkingDirectory"));
+
     QmakeProject *project = qmakeProject();
-    m_parseSuccess = project->validParse(m_proFilePath);
-    m_parseInProgress = project->parseInProgress(m_proFilePath);
-
-    ctor();
+    connect(project, &Project::parsingFinished,
+            this, &DesktopQmakeRunConfiguration::updateTargetInformation);
+    connect(project, &QmakeProject::proFilesEvaluated,
+            this, &DesktopQmakeRunConfiguration::proFileEvaluated);
 }
 
-DesktopQmakeRunConfiguration::DesktopQmakeRunConfiguration(Target *parent, DesktopQmakeRunConfiguration *source) :
-    RunConfiguration(parent, source),
-    m_proFilePath(source->m_proFilePath),
-    m_isUsingDyldImageSuffix(source->m_isUsingDyldImageSuffix),
-    m_isUsingLibrarySearchPath(source->m_isUsingLibrarySearchPath),
-    m_parseSuccess(source->m_parseSuccess),
-    m_parseInProgress(source->m_parseInProgress)
+void DesktopQmakeRunConfiguration::initialize(Core::Id id)
 {
-    ctor();
-}
+    RunConfiguration::initialize(id);
+    m_proFilePath = pathFromId(id);
 
-bool DesktopQmakeRunConfiguration::isEnabled() const
-{
-    return m_parseSuccess && !m_parseInProgress;
-}
-
-QString DesktopQmakeRunConfiguration::disabledReason() const
-{
-    if (m_parseInProgress)
-        return tr("The .pro file \"%1\" is currently being parsed.")
-                .arg(m_proFilePath.fileName());
-
-    if (!m_parseSuccess)
-        return qmakeProject()->disabledReasonForRunConfiguration(m_proFilePath);
-    return QString();
-}
-
-void DesktopQmakeRunConfiguration::proFileUpdated(QmakeProFileNode *pro, bool success, bool parseInProgress)
-{
-    if (m_proFilePath != pro->filePath())
-        return;
-    bool enabled = isEnabled();
-    QString reason = disabledReason();
-    m_parseSuccess = success;
-    m_parseInProgress = parseInProgress;
-    if (enabled != isEnabled() || reason != disabledReason())
-        emit enabledChanged();
-
-    if (!parseInProgress) {
-        emit effectiveTargetInformationChanged();
-        setDefaultDisplayName(defaultDisplayName());
-        extraAspect<LocalEnvironmentAspect>()->buildEnvironmentHasChanged();
-    }
+    updateTargetInformation();
 }
 
 void DesktopQmakeRunConfiguration::proFileEvaluated()
@@ -146,15 +108,21 @@ void DesktopQmakeRunConfiguration::proFileEvaluated()
     return extraAspect<LocalEnvironmentAspect>()->buildEnvironmentHasChanged();
 }
 
-void DesktopQmakeRunConfiguration::ctor()
+void DesktopQmakeRunConfiguration::updateTargetInformation()
 {
     setDefaultDisplayName(defaultDisplayName());
+    extraAspect<LocalEnvironmentAspect>()->buildEnvironmentHasChanged();
 
-    QmakeProject *project = qmakeProject();
-    connect(project, &QmakeProject::proFileUpdated,
-            this, &DesktopQmakeRunConfiguration::proFileUpdated);
-    connect(project, &QmakeProject::proFilesEvaluated,
-            this, &DesktopQmakeRunConfiguration::proFileEvaluated);
+    auto wda = extraAspect<WorkingDirectoryAspect>();
+
+    wda->setDefaultWorkingDirectory(FileName::fromString(baseWorkingDirectory()));
+    if (wda->pathChooser())
+        wda->pathChooser()->setBaseFileName(target()->project()->projectDirectory());
+    auto terminalAspect = extraAspect<TerminalAspect>();
+    if (!terminalAspect->isUserSet())
+        terminalAspect->setUseTerminal(isConsoleApplication());
+
+    emit effectiveTargetInformationChanged();
 }
 
 //////
@@ -167,17 +135,6 @@ DesktopQmakeRunConfigurationWidget::DesktopQmakeRunConfigurationWidget(DesktopQm
     auto vboxTopLayout = new QVBoxLayout(this);
     vboxTopLayout->setMargin(0);
 
-    auto hl = new QHBoxLayout();
-    hl->addStretch();
-    m_disabledIcon = new QLabel(this);
-    m_disabledIcon->setPixmap(Core::Icons::WARNING.pixmap());
-    hl->addWidget(m_disabledIcon);
-    m_disabledReason = new QLabel(this);
-    m_disabledReason->setVisible(false);
-    hl->addWidget(m_disabledReason);
-    hl->addStretch();
-    vboxTopLayout->addLayout(hl);
-
     auto detailsContainer = new DetailsWidget(this);
     detailsContainer->setState(DetailsWidget::NoSummary);
     vboxTopLayout->addWidget(detailsContainer);
@@ -233,9 +190,6 @@ DesktopQmakeRunConfigurationWidget::DesktopQmakeRunConfigurationWidget(DesktopQm
                 this, &DesktopQmakeRunConfigurationWidget::usingLibrarySearchPathToggled);
     }
 
-    runConfigurationEnabledChange();
-    effectiveTargetInformationChanged();
-
     connect(qmakeRunConfiguration, &DesktopQmakeRunConfiguration::usingDyldImageSuffixChanged,
             this, &DesktopQmakeRunConfigurationWidget::usingDyldImageSuffixChanged);
     connect(qmakeRunConfiguration, &DesktopQmakeRunConfiguration::usingLibrarySearchPathChanged,
@@ -243,18 +197,8 @@ DesktopQmakeRunConfigurationWidget::DesktopQmakeRunConfigurationWidget(DesktopQm
     connect(qmakeRunConfiguration, &DesktopQmakeRunConfiguration::effectiveTargetInformationChanged,
             this, &DesktopQmakeRunConfigurationWidget::effectiveTargetInformationChanged, Qt::QueuedConnection);
 
-    connect(qmakeRunConfiguration, &RunConfiguration::enabledChanged,
-            this, &DesktopQmakeRunConfigurationWidget::runConfigurationEnabledChange);
-
     Core::VariableChooser::addSupportForChildWidgets(this, m_qmakeRunConfiguration->macroExpander());
-}
-
-void DesktopQmakeRunConfigurationWidget::runConfigurationEnabledChange()
-{
-    bool enabled = m_qmakeRunConfiguration->isEnabled();
-    m_disabledIcon->setVisible(!enabled);
-    m_disabledReason->setVisible(!enabled);
-    m_disabledReason->setText(m_qmakeRunConfiguration->disabledReason());
+    effectiveTargetInformationChanged();
 }
 
 void DesktopQmakeRunConfigurationWidget::usingDyldImageSuffixToggled(bool state)
@@ -286,15 +230,6 @@ void DesktopQmakeRunConfigurationWidget::usingLibrarySearchPathChanged(bool stat
 void DesktopQmakeRunConfigurationWidget::effectiveTargetInformationChanged()
 {
     m_executableLineLabel->setText(QDir::toNativeSeparators(m_qmakeRunConfiguration->executable()));
-
-    m_ignoreChange = true;
-    auto aspect = m_qmakeRunConfiguration->extraAspect<WorkingDirectoryAspect>();
-    aspect->setDefaultWorkingDirectory(FileName::fromString(m_qmakeRunConfiguration->baseWorkingDirectory()));
-    aspect->pathChooser()->setBaseFileName(m_qmakeRunConfiguration->target()->project()->projectDirectory());
-    auto terminalAspect = m_qmakeRunConfiguration->extraAspect<TerminalAspect>();
-    if (!terminalAspect->isUserSet())
-        terminalAspect->setUseTerminal(m_qmakeRunConfiguration->isConsoleApplication());
-    m_ignoreChange = false;
 }
 
 QWidget *DesktopQmakeRunConfiguration::createConfigurationWidget()
@@ -330,16 +265,13 @@ bool DesktopQmakeRunConfiguration::fromMap(const QVariantMap &map)
     m_isUsingDyldImageSuffix = map.value(QLatin1String(USE_DYLD_IMAGE_SUFFIX_KEY), false).toBool();
     m_isUsingLibrarySearchPath = map.value(QLatin1String(USE_LIBRARY_SEARCH_PATH), true).toBool();
 
-    m_parseSuccess = qmakeProject()->validParse(m_proFilePath);
-    m_parseInProgress = qmakeProject()->parseInProgress(m_proFilePath);
-
     return RunConfiguration::fromMap(map);
 }
 
 QString DesktopQmakeRunConfiguration::executable() const
 {
-    if (QmakeProFileNode *node = projectNode())
-        return extractWorkingDirAndExecutable(node).second;
+    if (QmakeProFile *pro = proFile())
+        return extractWorkingDirAndExecutable(pro).second;
     return QString();
 }
 
@@ -371,16 +303,19 @@ void DesktopQmakeRunConfiguration::setUsingLibrarySearchPath(bool state)
 
 QString DesktopQmakeRunConfiguration::baseWorkingDirectory() const
 {
-    if (QmakeProFileNode *node = projectNode())
-        return extractWorkingDirAndExecutable(node).first;
+    if (QmakeProFile *pro = proFile())
+        return extractWorkingDirAndExecutable(pro).first;
     return QString();
 }
 
 bool DesktopQmakeRunConfiguration::isConsoleApplication() const
 {
-    if (QmakeProFileNode *node = projectNode()) {
-        return node->variableValue(ConfigVar).contains(QLatin1String("console"))
-                && !node->variableValue(QtVar).contains(QLatin1String("testlib"));
+    if (QmakeProFile *pro = proFile()) {
+        const QStringList config = pro->variableValue(Variable::Config);
+        if (!config.contains("console") || config.contains("testcase"))
+            return false;
+        const QStringList qt = pro->variableValue(Variable::Qt);
+        return !qt.contains("testlib") && !qt.contains("qmltest");
     }
     return false;
 }
@@ -393,11 +328,11 @@ void DesktopQmakeRunConfiguration::addToBaseEnvironment(Environment &env) const
     // The user could be linking to a library found via a -L/some/dir switch
     // to find those libraries while actually running we explicitly prepend those
     // dirs to the library search path
-    const QmakeProFileNode *node = projectNode();
-    if (m_isUsingLibrarySearchPath && node) {
-        const QStringList libDirectories = node->variableValue(LibDirectoriesVar);
+    const QmakeProFile *pro = proFile();
+    if (m_isUsingLibrarySearchPath && pro) {
+        const QStringList libDirectories = pro->variableValue(Variable::LibDirectories);
         if (!libDirectories.isEmpty()) {
-            const QString proDirectory = node->buildDir();
+            const QString proDirectory = pro->buildDir().toString();
             foreach (QString dir, libDirectories) {
                 // Fix up relative entries like "LIBS+=-L.."
                 const QFileInfo fi(dir);
@@ -406,13 +341,18 @@ void DesktopQmakeRunConfiguration::addToBaseEnvironment(Environment &env) const
                 env.prependOrSetLibrarySearchPath(dir);
             } // foreach
         } // libDirectories
-    } // node
+    } // pro
 
     QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(target()->kit());
     if (qtVersion && m_isUsingLibrarySearchPath)
         env.prependOrSetLibrarySearchPath(qtVersion->qmakeProperty("QT_INSTALL_LIBS"));
 }
 
+QString DesktopQmakeRunConfiguration::buildSystemTarget() const
+{
+    return qmakeProject()->mapProFilePathToTarget(m_proFilePath);
+}
+
 Utils::FileName DesktopQmakeRunConfiguration::proFilePath() const
 {
     return m_proFilePath;
@@ -423,19 +363,18 @@ QmakeProject *DesktopQmakeRunConfiguration::qmakeProject() const
     return static_cast<QmakeProject *>(target()->project());
 }
 
-QmakeProFileNode *DesktopQmakeRunConfiguration::projectNode() const
+QmakeProFile *DesktopQmakeRunConfiguration::proFile() const
 {
     QmakeProject *project = qmakeProject();
     QTC_ASSERT(project, return nullptr);
-    QmakeProFileNode *rootNode = project->rootProjectNode();
-    QTC_ASSERT(rootNode, return nullptr);
-    return rootNode->findProFileFor(m_proFilePath);
+    QmakeProFile *rootProFile = project->rootProFile();
+    return rootProFile ? rootProFile->findProFile(m_proFilePath) : nullptr;
 }
 
 QString DesktopQmakeRunConfiguration::defaultDisplayName()
 {
-    if (QmakeProFileNode *node = projectNode())
-        return node->displayName();
+    if (QmakeProFile *pro = proFile())
+        return pro->displayName();
 
     QString defaultName;
     if (!m_proFilePath.isEmpty())
@@ -450,32 +389,33 @@ OutputFormatter *DesktopQmakeRunConfiguration::createOutputFormatter() const
     return new QtSupport::QtOutputFormatter(target()->project());
 }
 
-QPair<QString, QString> DesktopQmakeRunConfiguration::extractWorkingDirAndExecutable(const QmakeProFileNode *node) const
+QPair<QString, QString> DesktopQmakeRunConfiguration::extractWorkingDirAndExecutable(const QmakeProFile *proFile) const
 {
-    if (!node)
-        return qMakePair(QString(), QString());
-    TargetInformation ti = node->targetInformation();
+    if (!proFile)
+        return {};
+
+    TargetInformation ti = proFile->targetInformation();
     if (!ti.valid)
         return qMakePair(QString(), QString());
 
-    const QStringList &config = node->variableValue(ConfigVar);
+    const QStringList &config = proFile->variableValue(Variable::Config);
 
-    QString destDir = ti.destDir;
+    QString destDir = ti.destDir.toString();
     QString workingDir;
     if (!destDir.isEmpty()) {
         bool workingDirIsBaseDir = false;
         if (destDir == ti.buildTarget)
             workingDirIsBaseDir = true;
         if (QDir::isRelativePath(destDir))
-            destDir = QDir::cleanPath(ti.buildDir + QLatin1Char('/') + destDir);
+            destDir = QDir::cleanPath(ti.buildDir.toString() + '/' + destDir);
 
         if (workingDirIsBaseDir)
-            workingDir = ti.buildDir;
+            workingDir = ti.buildDir.toString();
         else
             workingDir = destDir;
     } else {
-        destDir = ti.buildDir;
-        workingDir = ti.buildDir;
+        destDir = ti.buildDir.toString();
+        workingDir = ti.buildDir.toString();
     }
 
     if (HostOsInfo::isMacHost() && config.contains(QLatin1String("app_bundle"))) {
@@ -512,7 +452,7 @@ bool DesktopQmakeRunConfigurationFactory::canCreate(Target *parent, Core::Id id)
 
 RunConfiguration *DesktopQmakeRunConfigurationFactory::doCreate(Target *parent, Core::Id id)
 {
-    return new DesktopQmakeRunConfiguration(parent, id);
+    return createHelper<DesktopQmakeRunConfiguration>(parent, id);
 }
 
 bool DesktopQmakeRunConfigurationFactory::canRestore(Target *parent, const QVariantMap &map) const
@@ -524,7 +464,7 @@ bool DesktopQmakeRunConfigurationFactory::canRestore(Target *parent, const QVari
 
 RunConfiguration *DesktopQmakeRunConfigurationFactory::doRestore(Target *parent, const QVariantMap &map)
 {
-    return new DesktopQmakeRunConfiguration(parent, idFromMap(map));
+    return createHelper<DesktopQmakeRunConfiguration>(parent, idFromMap(map));
 }
 
 bool DesktopQmakeRunConfigurationFactory::canClone(Target *parent, RunConfiguration *source) const
@@ -536,8 +476,7 @@ RunConfiguration *DesktopQmakeRunConfigurationFactory::clone(Target *parent, Run
 {
     if (!canClone(parent, source))
         return 0;
-    DesktopQmakeRunConfiguration *old = static_cast<DesktopQmakeRunConfiguration *>(source);
-    return new DesktopQmakeRunConfiguration(parent, old);
+    return cloneHelper<DesktopQmakeRunConfiguration>(parent, source);
 }
 
 QList<Core::Id> DesktopQmakeRunConfigurationFactory::availableCreationIds(Target *parent, CreationMode mode) const
@@ -546,10 +485,7 @@ QList<Core::Id> DesktopQmakeRunConfigurationFactory::availableCreationIds(Target
         return QList<Core::Id>();
 
     QmakeProject *project = static_cast<QmakeProject *>(parent->project());
-    QList<QmakeProFileNode *> nodes = project->applicationProFiles();
-    if (mode == AutoCreate)
-        nodes = QmakeProject::nodesWithQtcRunnable(nodes);
-    return QmakeProject::idsForNodes(Core::Id(QMAKE_RC_PREFIX), nodes);
+    return project->creationIds(QMAKE_RC_PREFIX, mode);
 }
 
 QString DesktopQmakeRunConfigurationFactory::displayNameForId(Core::Id id) const
