@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -327,7 +322,7 @@ protected:
 
     virtual bool visit(AST::UiPublicMember *node)
     {
-        if (node->memberType == _name){
+        if (node->memberTypeName() == _name){
             const ObjectValue * tVal = _context->lookupType(_doc.data(), QStringList(_name));
             if (tVal == _typeValue)
                 _usages.append(node->typeToken);
@@ -588,8 +583,8 @@ protected:
     virtual bool visit(UiPublicMember *node)
     {
         if (containsOffset(node->typeToken)){
-            if (!node->memberType.isEmpty()) {
-                _name = node->memberType.toString();
+            if (node->isValid()) {
+                _name = node->memberTypeName().toString();
                 _targetValue = _scopeChain->context()->lookupType(_doc.data(), QStringList(_name));
                 _scope = 0;
                 _typeKind = TypeKind;
@@ -787,8 +782,8 @@ FindReferences::FindReferences(QObject *parent)
     : QObject(parent)
 {
     m_watcher.setPendingResultsLimit(1);
-    connect(&m_watcher, SIGNAL(resultsReadyAt(int,int)), this, SLOT(displayResults(int,int)));
-    connect(&m_watcher, SIGNAL(finished()), this, SLOT(searchFinished()));
+    connect(&m_watcher, &QFutureWatcherBase::resultsReadyAt, this, &FindReferences::displayResults);
+    connect(&m_watcher, &QFutureWatcherBase::finished, this, &FindReferences::searchFinished);
 }
 
 FindReferences::~FindReferences()
@@ -813,14 +808,14 @@ static void find_helper(QFutureInterface<FindReferences::Usage> &future,
         if (oldDoc && oldDoc->editorRevision() == it.value().second)
             continue;
 
-        Language::Enum language;
+        Dialect language;
         if (oldDoc)
             language = oldDoc->language();
         else
             language = ModelManagerInterface::guessLanguageOfFile(fileName);
-        if (language == Language::NoLanguage) {
+        if (language == Dialect::NoLanguage) {
             qCDebug(qmljsLog) << "NoLanguage in qmljsfindreferences.cpp find_helper for " << fileName;
-            language = Language::AnyLanguage;
+            language = Dialect::AnyLanguage;
         }
 
         Document::MutablePtr newDoc = snapshot.documentFromSource(
@@ -834,7 +829,7 @@ static void find_helper(QFutureInterface<FindReferences::Usage> &future,
     if (!doc)
         return;
 
-    QmlJS::ModelManagerInterface *modelManager = QmlJS::ModelManagerInterface::instance();
+    ModelManagerInterface *modelManager = ModelManagerInterface::instance();
 
     Link link(snapshot, modelManager->defaultVContext(doc->language(), doc), modelManager->builtins(doc));
     ContextPtr context = link();
@@ -896,10 +891,8 @@ void FindReferences::findUsages(const QString &fileName, quint32 offset)
 {
     ModelManagerInterface *modelManager = ModelManagerInterface::instance();
 
-    QFuture<Usage> result = QtConcurrent::run(
-                &find_helper, modelManager->workingCopy(),
-                modelManager->snapshot(), fileName, offset,
-                QString());
+    QFuture<Usage> result = Utils::runAsync(&find_helper, modelManager->workingCopy(),
+                                            modelManager->snapshot(), fileName, offset, QString());
     m_watcher.setFuture(result);
 }
 
@@ -913,13 +906,38 @@ void FindReferences::renameUsages(const QString &fileName, quint32 offset,
     if (newName.isNull())
         newName = QLatin1String("");
 
-    QFuture<Usage> result = QtConcurrent::run(
-                &find_helper, modelManager->workingCopy(),
-                modelManager->snapshot(), fileName, offset,
-                newName);
+    QFuture<Usage> result = Utils::runAsync(&find_helper, modelManager->workingCopy(),
+                                            modelManager->snapshot(), fileName, offset, newName);
     m_watcher.setFuture(result);
 }
 
+QList<FindReferences::Usage> FindReferences::findUsageOfType(const QString &fileName, const QString typeName)
+{
+    QList<Usage> usages;
+    ModelManagerInterface *modelManager = ModelManagerInterface::instance();
+
+    Document::Ptr doc = modelManager->snapshot().document(fileName);
+    if (!doc)
+        return usages;
+
+    Link link(modelManager->snapshot(), modelManager->defaultVContext(doc->language(), doc), modelManager->builtins(doc));
+    ContextPtr context = link();
+    ScopeChain scopeChain(doc, context);
+
+    const ObjectValue *targetValue = scopeChain.context()->lookupType(doc.data(), QStringList(typeName));
+
+    QmlJS::Snapshot snapshot =  modelManager->snapshot();
+
+    foreach (const QmlJS::Document::Ptr &doc, snapshot) {
+        FindTypeUsages findUsages(doc, context);
+        FindTypeUsages::Result results = findUsages(typeName, targetValue);
+        foreach (const AST::SourceLocation &loc, results) {
+            usages.append(Usage(doc->fileName(), matchingLine(loc.offset, doc->source()), loc.startLine, loc.startColumn - 1, loc.length));
+        }
+    }
+    return usages;
+}
+
 void FindReferences::displayResults(int first, int last)
 {
     // the first usage is always a dummy to indicate we now start searching
@@ -930,26 +948,28 @@ void FindReferences::displayResults(int first, int last)
         const QString label = tr("QML/JS Usages:");
 
         if (replacement.isEmpty()) {
-            m_currentSearch = Core::SearchResultWindow::instance()->startNewSearch(
-                        label, QString(), symbolName, Core::SearchResultWindow::SearchOnly);
+            m_currentSearch = SearchResultWindow::instance()->startNewSearch(
+                        label, QString(), symbolName, SearchResultWindow::SearchOnly);
         } else {
-            m_currentSearch = Core::SearchResultWindow::instance()->startNewSearch(
-                        label, QString(), symbolName, Core::SearchResultWindow::SearchAndReplace,
-                        Core::SearchResultWindow::PreserveCaseDisabled);
+            m_currentSearch = SearchResultWindow::instance()->startNewSearch(
+                        label, QString(), symbolName, SearchResultWindow::SearchAndReplace,
+                        SearchResultWindow::PreserveCaseDisabled);
             m_currentSearch->setTextToReplace(replacement);
-            connect(m_currentSearch, SIGNAL(replaceButtonClicked(QString,QList<Core::SearchResultItem>,bool)),
-                    SLOT(onReplaceButtonClicked(QString,QList<Core::SearchResultItem>,bool)));
+            connect(m_currentSearch.data(), &SearchResult::replaceButtonClicked,
+                    this, &FindReferences::onReplaceButtonClicked);
         }
-        connect(m_currentSearch, SIGNAL(activated(Core::SearchResultItem)),
-                this, SLOT(openEditor(Core::SearchResultItem)));
-        connect(m_currentSearch, SIGNAL(cancelled()), this, SLOT(cancel()));
-        connect(m_currentSearch, SIGNAL(paused(bool)), this, SLOT(setPaused(bool)));
-        Core::SearchResultWindow::instance()->popup(IOutputPane::Flags(IOutputPane::ModeSwitch | IOutputPane::WithFocus));
+        connect(m_currentSearch.data(), &SearchResult::activated,
+                [](const Core::SearchResultItem& item) {
+                    Core::EditorManager::openEditorAtSearchResult(item);
+                });
+        connect(m_currentSearch.data(), &SearchResult::cancelled, this, &FindReferences::cancel);
+        connect(m_currentSearch.data(), &SearchResult::paused, this, &FindReferences::setPaused);
+        SearchResultWindow::instance()->popup(IOutputPane::Flags(IOutputPane::ModeSwitch | IOutputPane::WithFocus));
 
         FutureProgress *progress = ProgressManager::addTask(
                     m_watcher.future(), tr("Searching for Usages"),
                     QmlJSEditor::Constants::TASK_SEARCH);
-        connect(progress, SIGNAL(clicked()), m_currentSearch, SLOT(popup()));
+        connect(progress, &FutureProgress::clicked, m_currentSearch.data(), &SearchResult::popup);
 
         ++first;
     }
@@ -987,17 +1007,7 @@ void FindReferences::setPaused(bool paused)
         m_watcher.setPaused(paused);
 }
 
-void FindReferences::openEditor(const Core::SearchResultItem &item)
-{
-    if (item.path.size() > 0) {
-        EditorManager::openEditorAt(QDir::fromNativeSeparators(item.path.first()),
-                                              item.lineNumber, item.textMarkPos);
-    } else {
-        EditorManager::openEditor(QDir::fromNativeSeparators(item.text));
-    }
-}
-
-void FindReferences::onReplaceButtonClicked(const QString &text, const QList<Core::SearchResultItem> &items, bool preserveCase)
+void FindReferences::onReplaceButtonClicked(const QString &text, const QList<SearchResultItem> &items, bool preserveCase)
 {
     const QStringList fileNames = TextEditor::BaseFileFind::replaceAll(text, items, preserveCase);
 
@@ -1012,9 +1022,9 @@ void FindReferences::onReplaceButtonClicked(const QString &text, const QList<Cor
     }
 
     if (!changedOnDisk.isEmpty())
-        QmlJS::ModelManagerInterface::instance()->updateSourceFiles(changedOnDisk, true);
+        ModelManagerInterface::instance()->updateSourceFiles(changedOnDisk, true);
     if (!changedUnsavedEditors.isEmpty())
-        QmlJS::ModelManagerInterface::instance()->updateSourceFiles(changedUnsavedEditors, false);
+        ModelManagerInterface::instance()->updateSourceFiles(changedUnsavedEditors, false);
 
-    Core::SearchResultWindow::instance()->hide();
+    SearchResultWindow::instance()->hide();
 }
