@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -32,13 +27,13 @@
 
 #include <coreplugin/icore.h>
 #include <utils/hostosinfo.h>
+#include <utils/synchronousprocess.h>
 
 #include <QSettings>
 #include <QLatin1String>
 #include <QLatin1Char>
 #include <QDir>
 #include <QFile>
-#include <QProcess>
 #include <QStringList>
 
 namespace TextEditor {
@@ -74,11 +69,12 @@ QString findFallbackDefinitionsLocation()
         QStringList programs;
         programs << QLatin1String("kde-config") << QLatin1String("kde4-config");
         foreach (const QString &program, programs) {
-            QProcess process;
-            process.start(program, QStringList(QLatin1String("--prefix")));
-            if (process.waitForStarted(5000)) {
-                process.waitForFinished(5000);
-                QString output = QString::fromLocal8Bit(process.readAllStandardOutput());
+            Utils::SynchronousProcess process;
+            process.setTimeoutS(5);
+            Utils::SynchronousProcessResponse response
+                    = process.runBlocking(program, QStringList(QLatin1String("--prefix")));
+            if (response.result == Utils::SynchronousProcessResponse::Finished) {
+                QString output = response.stdOut();
                 output.remove(QLatin1Char('\n'));
                 for (int i = 0; i < kateSyntaxCount; ++i) {
                     dir.setPath(output + kateSyntax[i]);
